/*
 * Copyright (c) 2015, 2019, Oracle and/or its affiliates. All rights reserved.
 *
 * This software is dual-licensed to you under the MIT License (MIT) and
 * the Universal Permissive License (UPL). See the LICENSE file in the root
 * directory for license terms. You may choose either license, or both.
 *
 */

this.config = {
    bundles : [
        // Device client library for NodeJS
        // The device client library for NodeJS is ECMAScript2015 (ES6) compatible.
        // These MUST be ordered such that classes are declared before they are accessed.
        {
            output: './build/modules/device-library.node.js',
            wrapper: './src/shared/wrapper-node-bundle.js',
            fields: {
                LIBNAME:'iotcs',
                DESCRIPTION:'Oracle IoT Cloud Service JavaScript Device Client Software Library',
                VERSION:'19.3.2.0.0-7',
                FILES: [
                    './src/device/Overview.js',
                    './src/device/Global.js',
                    './src/shared/PlatformNode.js',
                    './src/shared/RestApi.js',
                    './src/shared/StatusCode.js',
                    './src/shared/Impl.js',
                    './src/device/Mqtt.js',
                    './src/device/MqttController.js',
                    './src/device/Impl.js',
                    './src/shared/Client.js',
                    './src/shared/Monitor.js',
                    './src/shared/AbstractVirtualDevice.js',
                    './src/shared/UnifiedTrustStore.js',
                    './src/device/TrustedAssetsManager.js',
                    './src/device/Message.js',
                    './src/device/AlertMessage.js',
                    './src/device/ResourceMessage.js',
                    './src/device/ResourceMessageResource.js',
                    './src/device/DataItem.js',
                    './src/device/PersistenceMetaData.js',
                    './src/device/DeviceFunction.js',
                    './src/shared/DeviceModel.js',
                    './src/shared/DeviceModelFormat.js',
                    './src/shared/DeviceModelFormatField.js',
                    './src/device/DevicePolicyFunction.js',
                    './src/device/DevicePolicy.js',
                    './src/shared/DeviceModelParser.js',
                    './src/device/DeviceAnalog.js',
                    './src/device/DirectlyConnectedDeviceImpl.js',
                    './src/device/DirectlyConnectedDeviceUtil.js',
                    './src/device/FormulaParser.js',
                    './src/device/GatewayDeviceUtil.js',
                    './src/device/DeviceModelFactory.js',
                    './src/device/InMemoryPersistenceStore.js',
                    './src/device/TestConnectivity.js',
                    './src/device/MessageDispatcher.js',
                    './src/device/MessagePersistenceImplNode.js',
                    './src/device/BatchByPersistenceNode.js',
                    './src/device/RequestDispatcher.js',
                    './src/device/Attribute.js',
                    './src/shared/ActionSpecBase.js',
                    './src/device/ActionSpec.js',
                    './src/device/Alert.js',
                    './src/device/Data.js',
                    './src/device/DirectlyConnectedDevice.js',
                    './src/device/GatewayDevice.js',
                    './src/device/GatewayDeviceMetadata.js',
                    './src/shared/NamedValue.js',
                    './src/shared/ActionEvent.js',
                    './src/device/VirtualDevice.js',
                    './src/device/DevicePolicyManager.js',
                    './src/device/PersistenceStoreManager.js',
                    './src/device/PersistenceStoreTransaction.js',
                    './src/shared/ExternalObject.js',
                    './src/device/Stack.js',
                    './src/shared/StorageObject.js',
                    './src/device/StorageObject.js',
                    './src/device/StorageObjectSyncEvent.js',
                    './src/device/StorageObjectSyncStatus.js',
                    './src/shared/StorageDispatcher.js',
                    './src/shared/StorageDispatcherProgress.js',
                    './src/device/StorageDispatcher.js',
                    './src/device/StorageDispatcherProgress.js',
                    './src/device/StorageDispatcherProgressState.js',
                    './src/device/MessagingPolicyImpl.js',
                    './src/device/Bucket.js',
                    './src/shared/DeviceModelAction.js',
                    './src/shared/DeviceModelActionArgument.js',
                    './src/shared/DeviceModelAttribute.js',
                    './src/device/NetworkCost.js',
                    './src/device/Pair.js',
                    './src/device/ScheduledPolicyData.js',
                    './src/device/ScheduledPolicyDataKey.js',
                    './src/device/TimedPolicyThread.js',
                    './src/device/VirtualDeviceAttribute.js',
                    './src/device/WritableValue.js'
                ]
            }
        },

        // Device client src web browsers.
        // The device client src for web browsers is ECMAScript2015 (ES6) compatible.
        // These MUST be ordered such that classes are declared before they are accessed.
        {
            output: './build/modules/device-library.web.js',
            wrapper: './src/shared/wrapper-browser-bundle.js',
            fields: {
                LIBNAME:'iotcs',
                DESCRIPTION:'Oracle IoT Cloud Service JavaScript Device Client Software Library',
                VERSION:'19.3.2.0.0-7',
                FILES: [
                    './src/device/Overview.js',
                    './src/device/Global.js',
                    './src/shared/StatusCode.js',
                    './src/shared/PlatformBrowser.js',
                    './src/device/PlatformBrowser.js',
                    './src/shared/RestApi.js',
                    './src/shared/Impl.js',
                    './src/device/Mqtt.js',
                    './src/device/MqttController.js',
                    './src/device/Impl.js',
                    './src/shared/Client.js',
                    './src/shared/Monitor.js',
                    './src/shared/AbstractVirtualDevice.js',
                    './src/shared/UnifiedTrustStore.js',
                    './src/device/TrustedAssetsManager.js',
                    './src/device/Message.js',
                    './src/device/AlertMessage.js',
                    './src/device/ResourceMessage.js',
                    './src/device/ResourceMessageResource.js',
                    './src/device/DataItem.js',
                    './src/device/PersistenceMetaData.js',
                    './src/device/DeviceFunction.js',
                    './src/shared/DeviceModel.js',
                    './src/shared/DeviceModelFormat.js',
                    './src/shared/DeviceModelFormatField.js',
                    './src/device/DevicePolicyFunction.js',
                    './src/device/DevicePolicy.js',
                    './src/shared/DeviceModelParser.js',
                    './src/device/DeviceAnalog.js',
                    './src/device/DirectlyConnectedDeviceImpl.js',
                    './src/device/DirectlyConnectedDeviceUtil.js',
                    './src/device/FormulaParser.js',
                    './src/device/GatewayDeviceUtil.js',
                    './src/device/DeviceModelFactory.js',
                    './src/device/InMemoryPersistenceStore.js',
                    './src/device/TestConnectivity.js',
                    './src/device/MessageDispatcher.js',
                    './src/device/MessagePersistenceImplBrowser.js',
                    './src/device/BatchByPersistenceBrowser.js',
                    './src/device/RequestDispatcher.js',
                    './src/device/Attribute.js',
                    './src/shared/ActionSpecBase.js',
                    './src/device/ActionSpec.js',
                    './src/device/Alert.js',
                    './src/device/Data.js',
                    './src/device/DirectlyConnectedDevice.js',
                    './src/device/GatewayDevice.js',
                    './src/device/GatewayDeviceMetadata.js',
                    './src/shared/NamedValue.js',
                    './src/shared/ActionEvent.js',
                    './src/device/VirtualDevice.js',
                    './src/device/DevicePolicyManager.js',
                    './src/device/PersistenceStoreManager.js',
                    './src/device/PersistenceStoreTransaction.js',
                    './src/shared/ExternalObject.js',
                    './src/device/Stack.js',
                    './src/shared/StorageObject.js',
                    './src/device/StorageObject.js',
                    './src/device/StorageObjectSyncEvent.js',
                    './src/device/StorageObjectSyncStatus.js',
                    './src/shared/StorageDispatcher.js',
                    './src/shared/StorageDispatcherProgress.js',
                    './src/device/StorageDispatcher.js',
                    './src/device/StorageDispatcherProgress.js',
                    './src/device/StorageDispatcherProgressState.js',
                    './src/device/MessagingPolicyImpl.js',
                    './src/device/Bucket.js',
                    './src/shared/DeviceModelAction.js',
                    './src/shared/DeviceModelActionArgument.js',
                    './src/shared/DeviceModelAttribute.js',
                    './src/device/NetworkCost.js',
                    './src/device/Pair.js',
                    './src/device/ScheduledPolicyData.js',
                    './src/device/ScheduledPolicyDataKey.js',
                    './src/device/TimedPolicyThread.js',
                    './src/device/VirtualDeviceAttribute.js',
                    './src/device/WritableValue.js'
                ]
            }
        },

        // Enterprise client library for NodeJS.
        // The enterprise client library for NodeJS is ECMAScript2015 (ES6) compatible.
        // These MUST be ordered such that classes are declared before they are accessed.
        {
            output: './build/modules/enterprise-library.node.js',
            wrapper: './src/shared/wrapper-node-bundle.js',
            fields: {
                LIBNAME:'iotcs',
                DESCRIPTION:'Oracle IoT Cloud Service JavaScript Enterprise Client Software Library',
                VERSION:'19.3.2.0.0-7',
                FILES: [
                    './src/enterprise/Overview.js',
                    './src/enterprise/Global.js',
                    './src/shared/StatusCode.js',
                    './src/shared/Impl.js',
                    './src/shared/PlatformNode.js',
                    './src/shared/RestApi.js',
                    './src/shared/Client.js',
                    './src/shared/Monitor.js',
                    './src/shared/AbstractVirtualDevice.js',
                    './src/shared/UnifiedTrustStore.js',
                    './src/enterprise/Impl.js',
                    './src/shared/ActionSpecBase.js',
                    './src/shared/ExternalObject.js',
                    './src/enterprise/ActionSpec.js',
                    './src/enterprise/Alert.js',
                    './src/enterprise/Data.js',
                    './src/enterprise/Attribute.js',
                    './src/enterprise/AsyncRequestMonitor.js',
                    './src/enterprise/Controller.js',
                    './src/enterprise/Pageable.js',
                    './src/enterprise/MessageEnumerator.js',
                    './src/enterprise/ResourceEnumerator.js',
                    './src/enterprise/DeviceAppEnumerator.js',
                    './src/enterprise/EnterpriseClientImpl.js',
                    './src/enterprise/EnterpriseClient.js',
                    './src/enterprise/Filter.js',
                    './src/enterprise/Message.js',
                    './src/shared/DeviceModel.js',
                    './src/shared/DeviceModelAction.js',
                    './src/shared/DeviceModelActionArgument.js',
                    './src/shared/DeviceModelAttribute.js',
                    './src/shared/DeviceModelFormat.js',
                    './src/shared/DeviceModelFormatField.js',
                    './src/shared/DeviceModelParser.js',
                    './src/enterprise/Action.js',
                    './src/enterprise/VirtualDevice.js',
                    './src/enterprise/TrustedAssetsManager.js',
                    './src/shared/StorageDispatcher.js',
                    './src/shared/StorageDispatcherProgress.js',
                    './src/shared/StorageObject.js',
                    './src/enterprise/StorageObject.js',
                    './src/enterprise/StorageObjectSyncEvent.js',
                    './src/enterprise/StorageDispatcher.js'
                ]
            }
        },

        // Enterprise client library for web browsers.
        // The enterprise client library for web browsers is ECMAScript2015 (ES6) compatible.
        // These MUST be ordered such that classes are declared before they are accessed.
        {
            output: './build/modules/enterprise-library.web.js',
            wrapper: './src/shared/wrapper-browser-bundle.js',
            fields: {
                LIBNAME:'iotcs',
                DESCRIPTION:'Oracle IoT Cloud Service JavaScript Enterprise Client Software Library',
                VERSION:'19.3.2.0.0-7',
                FILES: [
                    './src/enterprise/Overview.js',
                    './src/enterprise/Global.js',
                    './src/shared/StatusCode.js',
                    './src/shared/PlatformBrowser.js',
                    './src/enterprise/PlatformBrowser.js',
                    './src/shared/RestApi.js',
                    './src/shared/Impl.js',
                    './src/shared/Client.js',
                    './src/shared/Monitor.js',
                    './src/shared/AbstractVirtualDevice.js',
                    './src/shared/UnifiedTrustStore.js',
                    './src/enterprise/Impl.js',
                    './src/shared/ActionSpecBase.js',
                    './src/shared/ExternalObject.js',
                    './src/enterprise/ActionSpec.js',
                    './src/enterprise/Alert.js',
                    './src/enterprise/Data.js',
                    './src/enterprise/Attribute.js',
                    './src/enterprise/AsyncRequestMonitor.js',
                    './src/enterprise/Controller.js',
                    './src/enterprise/Pageable.js',
                    './src/enterprise/MessageEnumerator.js',
                    './src/enterprise/ResourceEnumerator.js',
                    './src/enterprise/DeviceAppEnumerator.js',
                    './src/enterprise/EnterpriseClientImpl.js',
                    './src/enterprise/EnterpriseClient.js',
                    './src/enterprise/Filter.js',
                    './src/enterprise/Message.js',
                    './src/shared/DeviceModel.js',
                    './src/shared/DeviceModelAction.js',
                    './src/shared/DeviceModelActionArgument.js',
                    './src/shared/DeviceModelAttribute.js',
                    './src/shared/DeviceModelFormat.js',
                    './src/shared/DeviceModelFormatField.js',
                    './src/shared/DeviceModelParser.js',
                    './src/enterprise/Action.js',
                    './src/enterprise/VirtualDevice.js',
                    './src/enterprise/TrustedAssetsManager.js'
               ]
            }
        },

        // JavaScript bundle for Messaging API docs.
        {
            output: './build/temp/docs/iotcs.bundle.ll.web.js',
            wrapper: './src/shared/wrapper-browser-bundle.js',
            fields: {
                LIBNAME:'iotcs',
                DESCRIPTION:'Oracle IoT Cloud Service JavaScript Client Software Library Messaging API',
                VERSION:'19.3.2.0.0-7',
                FILES: [
                    './src/device/Global.js',
                    './src/device/Overview.js',
                    './src/device/DirectlyConnectedDeviceUtil.js',
                    './src/device/GatewayDeviceUtil.js',
                    './src/device/Message.js',
                    './src/device/AlertMessage.js',
                    './src/device/ResourceMessage.js',
                    './src/device/ResourceMessageResource.js',
                    './src/device/MessageDispatcher.js',
                    './src/device/RequestDispatcher.js',
                    './src/device/StorageDispatcher.js',
                    './src/device/StorageDispatcherProgress.js',
                    './src/device/StorageDispatcherProgressState.js',
                    './src/device/StorageObject.js',
                    './src/device/StorageObjectSyncStatus.js',
                    './src/device/TrustedAssetsManager.js',
                    './src/enterprise/Global.js',
                    './src/enterprise/DeviceAppEnumerator.js',
                    './src/enterprise/MessageEnumerator.js',
                    './src/enterprise/Pageable.js',
                    './src/enterprise/ResourceEnumerator.js',
                    './src/enterprise/TrustedAssetsManager.js',
                    './src/shared/ExternalObject.js',
                    './src/shared/StorageObject.js',
                    './src/shared/UnifiedTrustStore.js',
                    './src/shared/StatusCode.js'
                ]
            }
        },

        // JavaScript bundle for Virtualization API docs.
        {
            output: './build/temp/docs/iotcs.bundle.hl.web.js',
            wrapper: './src/shared/wrapper-browser-bundle.js',
            fields: {
                LIBNAME:'iotcs',
                DESCRIPTION:'Oracle IoT Cloud Service JavaScript Client Software Library Virtualization API',
                VERSION:'19.3.2.0.0-7',
                FILES: [
                    './src/device/Alert.js',
                    './src/device/Data.js',
                    './src/device/DirectlyConnectedDevice.js',
                    './src/device/GatewayDevice.js',
                    './src/device/GatewayDeviceMetadata.js',
                    './src/device/Global.js',
                    './src/device/StorageObject.js',
                    './src/device/StorageObjectSyncEvent.js',
                    './src/device/StorageObjectSyncStatus.js',
                    './src/device/VirtualDevice.js',
                    './src/enterprise/Global.js',
                    './src/enterprise/Overview.js',
                    './src/enterprise/Action.js',
                    './src/enterprise/AsyncRequestMonitor.js',
                    './src/enterprise/EnterpriseClient.js',
                    './src/enterprise/Filter.js',
                    './src/enterprise/Pageable.js',
                    './src/enterprise/VirtualDevice.js',
                    './src/shared/AbstractVirtualDevice.js',
                    './src/shared/ActionEvent.js',
                    './src/shared/Client.js',
                    './src/shared/ExternalObject.js',
                    './src/shared/StatusCode.js'
                ]
            }
        }
    ]
};
